{% extends "vendor/manage/base.html" %}
{% load i18n %}


{% block title %}Sites Payment Processors'{% endblock %}

{% block content %}
<div class='card'>
    <div class='card-header'>
        <h3>
            {% trans 'Processor Per Site' %}
        </h3>
    </div>
    <div class='card-body'>
        <table class="table">
            <thead>
                <tr>
                    <td>{% trans 'Site' %}</td>
                    <td>{% trans 'Processor' %}</td>
                    <td><a class="btn btn-primary" href="{% url 'vendor_admin:vendor-new-site-processor' %}">{% trans 'New' %}</a></td>
                </tr>
            </thead>
            <tbody>
                {% for object in object_list %}
                    <tr>
                        <td>{{object.site}}</td>
                        <td>{{object.value.payment_processor}}</td>
                        <td>
                            <a class="btn btn-success" href="{% url 'vendor_admin:vendor-sites-processors' object.site.domain %}">{% trans 'Edit Processor and Site' %}</a>
                        </td>
                    </tr>
                {% endfor %}
                
            </tbody>

        </table>
    </div>
</div>
{% endblock %}